# FixCon

## Introduction

This project is a Generic Solver for Fixed-Cardinality Subgraph Problems.

This means that for a given target-function *f* that maps graphs to the real numbers, a fixed natural number *k* and an
undirected graph *G*, it finds a connected subgraph of *G* with *k* vertices that maximizes the target-function *f*.  
As *f* may also be an indicator-function for a desired property of the subgraph (for example being a tree, containing no
triangles or being n-regular), the question of existance of subgraphs with this property can also be answered.

This is project of the [Algorithmics Research Group](https://www.uni-marburg.de/en/fb12/research-groups/algorith) of the
Philipps-Universität Marburg.

The paper the main algorithm is based off can be
found [here](https://www.uni-marburg.de/de/fb12/arbeitsgruppen/algorith/paper/alenex20-fixcon.pdf).

## Project structure

The project has 2 main packages: *core* and *fixcon*. The package *core* contains all classes etc. that might also be
reused for other purposes than FixCon and are having a more general character (like IO for graphs or a data structure to
manage set-partitionings).

The package *fixcon* has very specific classes like the heuristic for the objective function or a class whose only use
is tracking of the values for the Clique-Join-Rule.

The search-trees are created in the method *solve*. Each single search-tree chooses a vertex *v* of the input graph,
then enumerates all valid subgraphs containing *v*. After the search-tree is finished, *v* is then deleted. If there's
no optimal subgraph containing *v*, the process begins anew with a different vertex *v'*.

## Pruning Rules

## Glossary and abbreviations

- **CJR**: Clique Join Rule
- **closed** neighbourhood (of vertex *v*): Union of open neighbourhood of *v* and *v* itself.
- **edge-monotone**: Let *g'* be a graph created by adding edges, but no vertices to *g*. Then *f* is edge-monotone if *
  f*(*g'*) is not lower than *f*(*g*)
- **hereditary** property: If property *f* is true for graph *g*, it's also true for every subgraph of *g*.
- **open** neighbourhood (of vertex *v*): The set of all vertices that have an edge to some vertex *v*. Often written as
  N(*v*).
- **UGR**: Universal graph rule
- **VAR**: Vertex Addition Rule

## Libraries

In this project, the following libraries (located in the folder *lib*) are used:

- [JGraphT](https://jgrapht.org/) provides the datastructure for graphs
- [Commons Math](https://commons.apache.org/proper/commons-math/) Is used to sample from random distribution over a
  finite collection (where the elements don't all have the same probability). This is used in the heuristic, where new
  vertices are picked depending on their degree.
- [combinatoricslib](https://github.com/dpaukov/combinatoricslib) Is used for tasks like iterating through subsets of a
  fixed size.