package de.umr

import de.umr.core.printAnalysis
import de.umr.fixcon.AlgoStats
import de.umr.fixcon.GlobalData
import de.umr.fixcon.GlobalData.G
import de.umr.fixcon.GlobalData.edgeCount
import de.umr.fixcon.GlobalData.funcID
import de.umr.fixcon.GlobalData.funcParams
import de.umr.fixcon.GlobalData.k
import de.umr.fixcon.GlobalData.path
import de.umr.fixcon.GlobalData.timeout
import de.umr.fixcon.GlobalData.vertexCount
import java.io.File
import java.nio.file.Files.createDirectories
import java.nio.file.Paths

/**
 * Handles input from command-line, calls the algorithm and writes the result back into a file.
 */
fun main(args: Array<String>) {
    GlobalData.init(args)     // Reading of Command-Line arguments
    printAnalysis(G)

    val (solution, usedTime) = solve(timeout) // running the actual solver

    AlgoStats.print()

    createDirectories(Paths.get("results")) //logging results
    File("results/${File(args[0]).name}.$k.$funcID.fixcon")
        .writeText(
            "${funcID};${funcParams.joinToString(",")}".padStart(15) +
                    File(path).name.padStart(40) +
                    vertexCount.toString().padStart(7) +
                    edgeCount.toString().padStart(9) +
                    k.toString().padStart(4) +
                    usedTime.toString().padStart(14) +
                    solution.value.toString().padStart(6) +
                    ("Nodes: ${AlgoStats.searchTreeNodes}").padStart(20) +
                    "     " +
                    solution.subgraph.vertexSet().toString() +
                    "\n"
        )
}