package de.umr.core

import de.umr.core.dataStructures.addWeightedEdge
import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge
import org.jgrapht.graph.SimpleWeightedGraph
import org.jgrapht.util.VertexToIntegerMapping

/**
 * In the case that at some point the vertices in a graph aren't integers (e.g. Strings instead), it may be useful to
 * convert the vertices to integers. This won't change the results of the program, as the IDs of the vertices aren't used
 * anyway.
 *
 * *jGraphT* provides the method *getVertexToIntegerMapping*, which provides a Map from the current datatype to integers.
 * As far as know right now, they don't provide a method to create a new graph with int-IDs from the other graph.
 *
 * This function aims to provide that service.
 *
 * As *getVertexToIntegerMapping* has no specification about the said mapping, it is hard to test this method (I think).
 * One possibility would be to check for graph isomorphism (as the isomorphism class wouldn't change by renaming vertices).
 */
fun <V> mapToIntGraph(g: Graph<V, DefaultEdge>): Pair<SimpleWeightedGraph<Int, DefaultEdge>, VertexToIntegerMapping<V>> {
    val resultGraph = SimpleWeightedGraph<Int, DefaultEdge>(DefaultEdge::class.java)
    val mapping = org.jgrapht.Graphs.getVertexToIntegerMapping(g)

    for (edge in g.edgeSet())
        resultGraph.addWeightedEdge(mapping.vertexMap[g.getEdgeSource(edge)], mapping.vertexMap[g.getEdgeTarget(edge)], g.getEdgeWeight(edge))

    return resultGraph to mapping // returns both so that you could undo the operation later if wanted
}