package de.umr.core.dataStructures

import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge
import org.jgrapht.graph.SimpleWeightedGraph

/**This is a wrapper class containing a graph and an integer value.
 * They represent a subgraph of the original graph and the value the target function has on it.
 *
 * [subgraph] Is a [Graph] that is meant to be the subgraph the algorithm found as a (temporary) solution in the original graph.
 * [value] Stores *f([subgraph])* for the target function *f*. Storing this value saves us from repeated calculation.*/
class Solution<V>(var subgraph: Graph<V, DefaultEdge> = SimpleWeightedGraph(DefaultEdge::class.java), var value: Int = Int.MIN_VALUE) {

    /**pretty-print method*/
    override fun toString() = "Solution: size=${subgraph.vertexCount},  value=$value,  subgraph=${subgraph.vertexSet()}"

}