package de.umr.fixcon

import de.umr.fixcon.Settings.paddingRight

/**This object tracks statistics about the program while it runs.
 *
 * This is helpful to see how big the effect of the different reduction-rules is.
 * Further, the number of nodes in the underlying search-tree is a better measure
 * for complexity than runtime itself, as it is independent of hardware.*/
object AlgoStats {

    /**The number of nodes in the search-tree that's underlying in the algorithm.*/
    var searchTreeNodes: Long = 0

    /**The number of times the Vertex-Addition-Rule made a skip in the search-tree*/
    var vertexAdditionRuleSkips: Long = 0

    /**The number of times the Clique-Join-Rule made a skip in the search-tree*/
    var cliqueJoinRuleSkips: Long = 0

    /**The number of times the Critical-Twin-Rule made a skip in the search-tree*/
    var criticalTwinSkips: Long = 0

    /**The number of times the Universal-Graph-Rule made a skip in the search-tree*/
    var universalGraphRuleSkips: Long = 0

    /**The number of times the Domination-DAG-Rule made a skip in the search-tree*/
    var dominationDagSkips : Long = 0


    /**Prints an overview of all tracked statistics.*/
    fun print() {
        print("\nAlgorithm Statistics:\n${"=".repeat(50)}\n")
        println("Vertex-Addition-Rule skips:".padEnd(paddingRight) + vertexAdditionRuleSkips)
        println("Clique-Join-Rule skips:".padEnd(paddingRight) + cliqueJoinRuleSkips)
        println("Critical-Twins skips:".padEnd(paddingRight) + criticalTwinSkips)
        println("Universal-Graph-Rule skips".padEnd(paddingRight) + universalGraphRuleSkips)
        println("Domination-DAG skips".padEnd(paddingRight) + dominationDagSkips)
        println("Search-Tree Nodes:".padEnd(paddingRight) + searchTreeNodes)
    }
}