package de.umr.fixcon

import de.umr.core.dataStructures.*
import de.umr.fixcon.GlobalData.f
import de.umr.fixcon.GlobalData.k
import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge
import org.paukov.combinatorics3.Generator


/**All pruning-rules that can be cleanly encapsulated are stored in this file.
 * Note that some other rules may be elsewhere, probably in the "solve" method in Main.*/


/**Returns *true* if it is not possible to expand [curr] with new vertices so that its value is better than [currentBestSol].
 * In this case, we can backtrack the algorithm and search elsewhere.*/
fun <V> vertexAdditionRule(curr: Graph<V, DefaultEdge>, currentBestSol: Solution<V>): Boolean {
    return if (f.isEdgeWeighted) {
        false
        //TODO anpassen
    } else {
        val currIsHopeless: Boolean = f.eval(curr) + f.completeBound(curr) <= currentBestSol.value
        if (currIsHopeless) AlgoStats.vertexAdditionRuleSkips++
        currIsHopeless
    }
}

/**Only works on subgraphs of size k-1. It checks every possible extension of the subgraph and if all of them don't
 * lead to a solution than the current best, you know you can backtrack. The reason is that the value of [f] only depends
 * on the isomorphism-class of graphs. This way you already tested all possible extensions of the subgraph.
 *
 */
fun universalGraphRule(sub: OrderedGraph<Int>, sol: Int, vList: List<Int> = sub.vertexSet().toList()): Boolean {
    if (f.isEdgeWeighted) {
        return false
        //TODO anpassen
    } else {

        if (k - sub.vertexCount != 1) return false //only works with one vertex missing
        var beaten = false
        sub.addVertex(Int.MIN_VALUE) //assumes this vertex isn't taken yet
        for (toggleSet in Generator.subset(vList).simple().stream().skip(1)) {    //skips first because it's the empty subset which would disconnect that graph
            sub.disconnectVertices(setOf(Int.MIN_VALUE), vList)
            sub.connectVertices(setOf(Int.MIN_VALUE), toggleSet)
            if (f.eval(sub) > sol) {
                beaten = true
                break
            }
        }

        sub.removeLastVertex()
        return !beaten.also { if (it) AlgoStats.universalGraphRuleSkips++ }
    }
}