package de.umr.fixcon.objectiveFunctions

import de.umr.fixcon.GlobalData
import org.jgrapht.Graph
import org.jgrapht.GraphMetrics
import org.jgrapht.graph.DefaultEdge

/**This function aims to maximize the diameter of the graph.
 *
 *The diameter has the following definition: For all pairs of vertices of the graph,
 *compute the shortest path between them. From the lengths of all these paths, take the minimum.
 *More compact: The diameter of a graph is its longest of all shortest paths between any two vertices.*/
class DiameterFunction : AbstractGraphFunction() {

    override val allowsTwinRule: Boolean get() = true

    /**If *p_i* are all the longest paths, then the new vertex *v* is added to one end of each *p_i*
     * in such a way that the length from *v* to the other end of *p_i* is one longer than the
     * current diameter.
     *
     * Thus, the diameter of the whole graph is increased by 1.*/
    override val vertexAdditionBound: Int get() = 1

    /**The diameter of [g], which is the longest shortest path in [g], or equivalently the
     * maximum of all the distances between any two vertices in [g].*/
    override fun <V> eval(g: Graph<V, DefaultEdge>) = GraphMetrics.getDiameter(g).toInt()

    /**Path with *k* vertices has *k*-1 edges that form a path.*/
    override fun globalOptimum() = GlobalData.k - 1
}