package de.umr.fixcon.objectiveFunctions

import de.umr.core.dataStructures.edgeCount
import de.umr.core.dataStructures.vertexCount
import de.umr.fixcon.GlobalData
import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge

/**This function counts the number of edges in a given graph.*/
class EdgeCountFunction : AbstractGraphFunction() {

    override val allowsTwinRule: Boolean get() = true

    override val edgeMonotone = true

    /**corresponds to the arithmetic series: subgraph.size + subgraph.size+1 + ... + targetSize-1*/
    override fun <V> completeBound(subgraph: Graph<V, DefaultEdge>) = (subgraph.vertexCount until GlobalData.k).sum()

    /**The number of edges in [g].*/
    override fun <V> eval(g: Graph<V, DefaultEdge>): Int = g.edgeCount

    /**The optimum would be a clique of size *k*.
     * There every one of the *k* vertices has a degree of [k-1]. By the handshaking lemma,
     * the number of edges is half the sum of all degrees.*/
    override fun globalOptimum() = GlobalData.k * (GlobalData.k - 1) / 2
}