package de.umr.fixcon.objectiveFunctions

import de.umr.core.dataStructures.degrees
import de.umr.fixcon.GlobalData
import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge

/**Returns the minimum degree of all vertices in the subgraph.*/
class MinDegreeFunction : AbstractGraphFunction() {

    override val allowsTwinRule: Boolean get() = true

    /**More edges, higher degrees...*/
    override val edgeMonotone = true

    /**Potentially, the new vertex could increase the degree of each vertex
     * with minimal degree by 1.*/
    override val vertexAdditionBound: Int get() = 1

    /**@return The minimum of the degrees of all vertices in [g].*/
    override fun <V> eval(g: Graph<V, DefaultEdge>) = g.degrees.minOrNull()!!

    /**The optimal graph is a clique of size *k*, in which each vertex has a degree of *k*-1.*/
    override fun globalOptimum() = GlobalData.k - 1
}