package de.umr.fixcon.objectiveFunctions

import de.umr.core.dataStructures.degrees
import de.umr.fixcon.GlobalData
import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge

/**The goal is to minimize the maximal degree (so the best possible connected graph for that is a tree).
 * To model a minimization problem, we maximize the negative of the function.
 *
 *
 * @return the maximum degree of all vertices in this graph.*/
class NegMaxDegreeFunction : AbstractGraphFunction() {

    override val allowsTwinRule: Boolean get() = true

    /**The maximum degree cannot be reduced by adding vertices. Therefore, the best case
     * is if the maximum degree is not changed at all.*/
    override fun <V> completeBound(subgraph: Graph<V, DefaultEdge>) = 0

    /**@return The negative of the maximum degree of the vertices in [g].*/
    override fun <V> eval(g: Graph<V, DefaultEdge>) = -g.degrees.maxOrNull()!!

    /**Assuming the graph is connected and has a size of at least 3, there needs to be
     * a vertex with degree of at least 2 to connect the other vertices.*/
    override fun globalOptimum() = if (GlobalData.k <= 2) -1 else -2
}