package de.umr.fixcon.objectiveFunctions

import de.umr.fixcon.DegreeConstrainedFunction
import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge

/**This method is a encoding for the decision-problem if a graph is *R-regular*.
 * A graph is *R-regular* if every vertex of it has exactly *R* neighbours.
 *
 * This function returns 1 (indicator for **True) if graph is *R-regular*.*/
class RRegularFunction(params: List<Int>) : AbstractGraphFunction(params) {

    override val allowsTwinRule: Boolean get() = true

    /**The number of vertices whose degree is smaller than the lower bound, but could be increased
     * enough to fit into the range given by [params] with the vertices that can yet be added
     * to the subgraph.*/
    override fun <V> completeBound(subgraph: Graph<V, DefaultEdge>) = DegreeConstrainedFunction(params).completeBound(subgraph)

    /**Counts the number of invalid vertices and negates the number,
     * so that the function increases as fewer vertices are invalid.*/
    override fun <V> eval(g: Graph<V, DefaultEdge>) = DegreeConstrainedFunction(listOf(params[0], params[0])).eval(g)

    /**If no vertex has the wrong degree, the functional value is 0*/
    override fun globalOptimum() = 0
}