package de.umr.fixcon.objectiveFunctions

import de.umr.core.dataStructures.hasTriangle
import org.jgrapht.Graph
import org.jgrapht.graph.DefaultEdge

/**This function models the decision-problem if a graph is triangle-free.
 * It returns 0 if the graph contains a triangle, and 1 if not.
 *
 * This function is hereditary (if *x* is a triangle-free graph, then every subgraph of *x* is also triangle-free).
 * */
class TriangleFreeFunction : AbstractGraphFunction() {

    override val allowsTwinRule: Boolean get() = true

    /***jGraphT* also provides the function *isTriangleFree*, but it counts all triangles for this and compares that with 0.
     * In most cases, this takes significantly longer. Therefore, the extension-method [hasTriangle] is used.
     *
     * @return 0 if [g] contains a triangle else 1.*/
    override fun <V> eval(g: Graph<V, DefaultEdge>) = if (g.hasTriangle()) 0 else 1

    /**The value 1 indicates *True*.*/
    override fun globalOptimum() = 1
}